/*
 * Copyright (C) 2013 by the University of Southern California
 * $Id: query.cc 20021 2014-02-20 21:50:43Z liangzhu $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "query.hh"

void print_section(ns_msg handle, int count, ns_sect sec_t, const char* sec_name)
{
	ns_rr rr;	
	char buf[1500];
	cout << ";; "<< sec_name <<" SECTION:" << endl;
	for (int rrnum=0; rrnum<count; rrnum++) {
		memset(&rr, 0, sizeof(rr));
		if (ns_parserr(&handle, sec_t, rrnum, &rr)) {
			cerr<<"[ERROR]ns_parserr"<<endl;
			exit(-1);
		}
		memset(buf, '\0', sizeof(buf));
		ns_sprintrr (&handle, &rr, NULL, NULL, buf, sizeof (buf));
		cout << buf << endl;
	}	
}

u_int16_t get_dns_id(unsigned char* msg, int msg_len)
{
	ns_msg 	handle;
	if (ns_initparse(msg, msg_len, &handle) < 0) {
		perror("ns_initparse");
		exit(-1);
	}
	u_int16_t ID      = ns_msg_id(handle);
	return ID;
}

void put_dns_id(unsigned char* msg, u_int16_t id)
{
	ns_put16(id, msg);	
}

void print_dns_msg(unsigned char* msg, int msg_len)
{
	ns_msg 	handle;
	if (ns_initparse(msg, msg_len, &handle) < 0) {
		perror("ns_initparse");
		exit(-1);
	}
	
	u_int16_t ID      = ns_msg_id(handle);
	u_int16_t QR      = ns_msg_getflag(handle, ns_f_qr);
	u_int16_t AA      = ns_msg_getflag(handle, ns_f_aa);
	u_int16_t TC      = ns_msg_getflag(handle, ns_f_tc);
	u_int16_t RD      = ns_msg_getflag(handle, ns_f_rd);
	u_int16_t RA      = ns_msg_getflag(handle, ns_f_ra);
	u_int16_t Z       = ns_msg_getflag(handle, ns_f_z);
	u_int16_t Opcode  = ns_msg_getflag(handle, ns_f_opcode);
	u_int16_t Rcode   = ns_msg_getflag(handle, ns_f_rcode);
	u_int16_t QdCount = ns_msg_count(handle, ns_s_qd);
	u_int16_t AnCount = ns_msg_count(handle, ns_s_an);
	u_int16_t NsCount = ns_msg_count(handle, ns_s_ns);
	u_int16_t ArCount = ns_msg_count(handle, ns_s_ar);

	cout << "\n----DNS Msg: len=" << msg_len << "; "<< (QR==0 ? "query" : "response") << "; id="<< ID << "; RD=" << RD <<"----" << endl;
	cout << ";; QdCount=" << QdCount << "; AnCount="<< AnCount << "; NsCount="<< NsCount << "; ArCount=" << ArCount << endl;
	print_section(handle, QdCount, ns_s_qd, "QUESTION");
	print_section(handle, AnCount, ns_s_an, "ANSWER");
	print_section(handle, NsCount, ns_s_ns, "AUTHORITY");
	print_section(handle, ArCount, ns_s_ar, "ADDITIONAL");	
}

bool is_tls_set (unsigned char* msg, int msg_len)
{
	LOG( LOG_DEBUG, "[DEBUG] is_tls_set()\n");
	ns_msg 	handle;
	if (ns_initparse(msg, msg_len, &handle) < 0) {
		perror("ns_initparse");
		exit(-1);
	}
	u_int16_t ArCount = ns_msg_count(handle, ns_s_ar);
	if (ArCount == 0)//no OPT
		return false;

	ns_rr rr;	
	
	//find the first query name: this should be '.'	
	memset(&rr, 0, sizeof(rr));
	if (ns_parserr(&handle, ns_s_qd, 0, &rr)) {
		die ("ns_parserr");
	}
	string rr_name = ns_rr_name(rr);
	LOG( LOG_DEBUG, "[DEBUG] is_tls_set(): query name is %s\n", rr_name.c_str());
		
	//EDNS with tls bit set?
	u_int16_t rr_type = 0;
	u_int32_t rr_ttl = 0;
	for (int rrnum=0; rrnum<ArCount; rrnum++) {
		memset(&rr, 0, sizeof(rr));
		if (ns_parserr(&handle, ns_s_ar, rrnum, &rr)) {
			die ("[ERROR] ns_parserr");
		}
		//ns_sprintrr (&handle, &rr, NULL, NULL, buf, sizeof (buf));
		rr_type = ns_rr_type(rr);
		rr_ttl = ns_rr_ttl(rr);
		if (rr_type == ns_t_opt) {//OPT, ENDS
			if (rr_ttl & DNS_MESSAGEEXTFLAG_TLS) {//tls bit is set
				LOG( LOG_DEBUG, "[DEBUG] tls bit is set\n");
				LOG( LOG_DEBUG, "[DEBUG] TTL bits:0x%08x\n",rr_ttl);
				return true;
			}
		}
	}
	return false;
}

unsigned char* tcp_pull_dns_pkt(int s, int &dns_pkt_size)
{
	unsigned char* size_info = new unsigned char[2];
	int n = read_byte(s, size_info, 2);
	if (n == 0) {
		dns_pkt_size = n;
		return NULL;
	}
	if (n != 2) {
		cerr << "n="<<n <<endl; 
		die("tcp_pull_dns_pkt: read dns size_info error");
	}
	u_int16_t dns_size = 0;
	memcpy(&dns_size, size_info, sizeof(size_info));
	dns_size = ntohs(dns_size);
	dns_pkt_size = dns_size + 2;
	unsigned char* real_info = new unsigned char[dns_size];
	n = read_byte(s, real_info, dns_size);
	if (n != dns_size) {
		cerr << "n="<<n <<endl; 
		die("tcp_pull_dns_pkt: read dns pkt error");
	}
	unsigned char* dns_pkt = new unsigned char[dns_pkt_size];
	memcpy (dns_pkt, size_info, 2);
	memcpy (dns_pkt + 2, real_info, dns_size);
	//delete[] header;
	delete[] size_info;
	delete[] real_info;
	return dns_pkt;
}

#ifdef USE_GNUTLS
unsigned char* tls_pull_dns_pkt(gnutls_session_t s, int &dns_pkt_size)
{
	unsigned char* size_info = new unsigned char[2];
	int n = tls_read_byte(s, size_info, 2);
	if (n == 0) {
		dns_pkt_size = n;
		return NULL;
	}
	if (n != 2) {
		cerr << "n="<<n <<endl; 
		die("tls_pull_dns_pkt: read dns size_info error");
	}
	u_int16_t dns_size = 0;
	memcpy(&dns_size, size_info, sizeof(size_info));
	dns_size = ntohs(dns_size);
	dns_pkt_size = dns_size + 2;
	unsigned char* real_info = new unsigned char[dns_size];
	n = tls_read_byte(s, real_info, dns_size);
	if (n != dns_size) {
		cerr << "n="<<n <<endl; 
		die("tls_pull_dns_pkt: read dns pkt error");
	}
	unsigned char* dns_pkt = new unsigned char[dns_pkt_size];
	memcpy (dns_pkt, size_info, 2);
	memcpy (dns_pkt + 2, real_info, dns_size);
	//delete[] header;
	delete[] size_info;
	delete[] real_info;
	return dns_pkt;
}
#endif //#ifdef USE_GNUTLS

unsigned char* udp_pull_dns_pkt(int s, int &dns_pkt_size, struct sockaddr* addr, socklen_t* addr_len)
{
	unsigned char* buf = new unsigned char[NS_PACKETSZ];//max udp 512 bytes
	
	int recv_len = recvpkt_from(s, buf, NS_PACKETSZ, addr, addr_len);
	dns_pkt_size = recv_len;
	if (recv_len == 0 || recv_len == -1) {
		return NULL;
	}
	return buf;
}
unsigned char* make_tcp_pkt(unsigned char *buf, int &size)
{
	unsigned char* tcp_pkt = new unsigned char[size + 2];
	u_int16_t len_to_write = htons(size);//this is very important
	memcpy(tcp_pkt, &len_to_write, sizeof(len_to_write));
	memcpy (tcp_pkt + 2, buf, size);
	size += 2;
	return tcp_pkt;
}












