/*
 * Copyright (C) 2013 by the University of Southern California
 * $Id: socket_util.cc 19922 2014-02-05 22:39:40Z liangzhu $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "socket_util.hh"

/*
	send
*/
int sendpkt(int s, unsigned char *buf, int len)
{
        ssize_t send_bytes = send(s, buf, len, 0);
        if (send_bytes == -1) {
                perror("sendpkt");
        }
        return send_bytes;
}

/*
	send
*/
int sendpkt_to(int s, unsigned char *buf, int len, struct sockaddr* addr, socklen_t addr_len)
{
	
	ssize_t send_bytes = sendto(s, buf, len, 0, addr, addr_len);
	if (send_bytes == -1) {
		perror("sendpkt_to:");
	
	}
        return send_bytes;
}

/*
	recv
*/
int recvpkt(int s, unsigned char *buf, int len)
{
        ssize_t recv_bytes = recv(s, buf, len, 0);
        if (recv_bytes == -1) {
                perror("recvpkt");      	
        } 
        return recv_bytes;
}


/*
	recv
*/
int recvpkt_from(int s, unsigned char *buf, int len, struct sockaddr* addr, socklen_t* addr_len)
{
	//struct sockaddr_in remote_addr; 
       // socklen_t addr_len = sizeof(remote_addr);
        ssize_t recv_bytes = recvfrom(s, buf, len, 0, addr, addr_len);
        if (recv_bytes == -1) {
                perror("recvpkt_from");      	
        } 
        return recv_bytes;
}

/*
	read bytes
*/
int read_byte(int s, unsigned char *buf, int len)
{
	unsigned char *buf2 = new unsigned char[len]; 
	int n = -1;
	int have_read = 0;
	int to_read = len;
	while (have_read < len) {
		memset(buf2, 0, len);
		n = read(s, buf2, to_read);
		if (n == -1) {
		        perror("read_byte");
		        return n;     	
		} 
		if (n == 0) {
			return 0;
		}
		if (n == to_read) {
			memcpy(buf + have_read, buf2, n);
			have_read += n;
			to_read -=n;
			return have_read;
		}
		memcpy(buf + have_read, buf2, n);
		have_read += n;
		to_read -=n;
	}
	return have_read;
}


/*
	make a connection either for udp or tcp
*/
int mk_conn (const char *ip, int port, bool tcp)
{
	struct sockaddr_in sa;	
	int fd = -1;
	if ((fd = socket(AF_INET, tcp ? SOCK_STREAM : SOCK_DGRAM, 0)) == -1) {
		perror("socket");
		exit(-1);
	}
	memset (&sa, '\0', sizeof (sa));
	sa.sin_family = AF_INET;
	sa.sin_port = htons (port);
	inet_pton (AF_INET, ip, &sa.sin_addr);
	if (connect (fd, (struct sockaddr *) & sa, sizeof (sa)) < 0) {
		perror("connect");
		exit(-1);
	}	
	return fd;
}

/*
	connect to server over tcp
*/
int tcp_connect (int port, const char *ip)
{

        struct sockaddr_in serv_addr;
        int tcp_socket = 0;
        if ((tcp_socket = socket(AF_INET,SOCK_STREAM,0)) == -1) {
                perror("socket");
                exit(1);
        }

        memset(&serv_addr,0,sizeof(struct sockaddr_in));
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = inet_addr(ip);
        serv_addr.sin_port = htons(port);

        //connect to the server
        int status = connect(tcp_socket,(struct sockaddr *)&serv_addr,sizeof(serv_addr));
        if (status < 0) {
                perror("connect");
                exit(1);
        }
        return tcp_socket;
}

/*
	connect to server over udp
*/
int udp_connect (int port, const char *ip)
{

        struct sockaddr_in serv_addr;
        int udp_socket = 0;
        if ((udp_socket = socket(AF_INET,SOCK_DGRAM,0)) == -1) {
                perror("socket");
                exit(1);
        }

        memset(&serv_addr,0,sizeof(struct sockaddr_in));
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = inet_addr(ip);
        serv_addr.sin_port = htons(port);

        //connect to the server
        int status = connect(udp_socket,(struct sockaddr *)&serv_addr,sizeof(serv_addr));
        if (status < 0) {
                perror("connect");
                exit(1);
        }
        return udp_socket;
}
