/*
 * Copyright (C) 2013 by the University of Southern California
 * $Id: utility.cc 19922 2014-02-05 22:39:40Z liangzhu $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "utility.hh"

void trim_spaces(string &s) {
	stringstream trimmer;
	trimmer << s;
	s.clear();
	trimmer >> s;	
}

/*
	print err msg and exit
*/
void die (const char *s)
{
	cerr << "[ERROR] " << s << endl;
	exit(-1);
}

/*
	print a warning
*/
void warning (const char *s)
{
	cerr << "[Warning] " << s << endl;
}

/*
	convert int to string
*/
void int2str (int n, string& s)
{
	ostringstream convert;
	convert << n;
	s = convert.str();
}

/*
	test a string is number or not
*/
bool is_number(string& s)
{
	if (s.empty()) return false;
	bool firsttime = true;
	string::iterator it;
	for(it = s.begin() ; it < s.end(); it++) {
		if (firsttime) {	//skip the '-' for negtive number 
			if (*it == '-') continue;
			firsttime = false;
		}
		if(!isdigit(*it)) return false;
	}
	return true;
}

/*
	check path and output errors, a file or a directory
*/
void check_path(char *f, bool is_dir)
{
	if (strlen(f) > 0) {
    		struct stat statbuf;
		if (stat(f, &statbuf) == -1) {  //get the property of the file
			cerr << "[ERROR] \"" << f << "\" ";
			perror("cannot be opened");
			exit(-1);
        	}
        	if (is_dir) {	//should be a directory
			if(!S_ISDIR(statbuf.st_mode)) {
				cerr << "[ERROR] \"" << f << "\" is a not directory" << endl;
				exit(-1);
			}        		
        	} else {	//should be a file
			if(S_ISDIR(statbuf.st_mode)) {
				cerr << "[ERROR] \"" << f << "\" is a directory" << endl;
				exit(-1);
			} 
        	}
	} else {
		cerr << "[ERROR] \"" << f <<" \": path is not valid." << endl;
		exit(-1);
	}
}

/*
	split a string
*/
void str_split(string &s, vector<string> &substrs)
{	
	if (s == "") {
		return;
	}
	istringstream ss(s);
	while(!ss.eof()) {//remove previous four items
		string x;
		ss >> x;
		substrs.push_back(x);
	}
}

/*
	get time
*/
double get_time_now(struct timeval *tv, struct timezone *tz, string t){
	if (gettimeofday(tv, tz) != 0) {
		die("gettimeofday fails!");
	}
	if (t== "second") {
		return (double(tv->tv_sec) + double(tv->tv_usec) / 1000000);
	} else if (t== "ms") {
		return (double(tv->tv_sec) * 1000  + double(tv->tv_usec) / 1000);
	} else if (t== "us") {
		return (double(tv->tv_sec) * 1000000  + double(tv->tv_usec));
	} else {
		cerr << "[ERROR] \"t\" is not supported in get_time_now"<<endl; 
		return -1;
	}
}

double get_time_now(string t){
	struct timeval tv;
	if (gettimeofday(&tv, NULL) != 0) {
		die("gettimeofday fails!");
	}
	if (t== "second") {
		return (double(tv.tv_sec) + double(tv.tv_usec) / 1000000);
	} else if (t== "ms") {
		return (double(tv.tv_sec) * 1000  + double(tv.tv_usec) / 1000);
	} else if (t== "us") {
		return (double(tv.tv_sec) * 1000000  + double(tv.tv_usec));
	} else {
		cerr << "[ERROR] \"t\" is not supported in get_time_now"<<endl; 
		return -1;
	}
}

